==> g-language-1.6.13.tar.gz: g-language-1.6.13/lib/G/Shell/Help.pm <==
#!/usr/bin/env perl

#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# This file is part of G-language Genome Analysis Environment package
#
#     Copyright (C) 2001-2007 Keio University
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# 
#   $Id: Skyline.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
#
# G-language GAE is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
# 
# G-language GAE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with G-language GAE -- see the file COPYING.
# If not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
#END_HEADER
#
# written by Kazuharu Arakawa <gaou@sfc.keio.ac.jp> at
# G-language Project, Institute for Advanced Biosciences, Keio University.
#

package G::Shell::Help;

use strict;
use File::Find;
use String::Approx 'amatch';

require Exporter;

our @ISA = qw(Exporter);

our @EXPORT = qw(
		 help
		 buildHelpDB
	     );

#:::::::::::::::::::::::::::::::::
#       Perldoc
#:::::::::::::::::::::::::::::::::


=head1 NAME

  G::Shell::Help - G-language Shell helper module for command references

=head1 DESCRIPTION

  This class is a part of G-language Genome Analysis Environment, 
  providing functions for "help" features of the G-language Shell.

=cut

#::::::::::::::::::::::::::::::
#          Variables
#::::::::::::::::::::::::::::::

my %pdoc;

#::::::::::::::::::::::::::::::
#    Let the code begin...
#::::::::::::::::::::::::::::::


sub buildHelpDB{
    my $bpPath = qx/perldoc -l Bio::Perl/;
    my $gPath = qx/perldoc -l G::Skyline/;
    my $rcmdPath = qx/perldoc -l Rcmd::Clustering/;
    
    $bpPath =~ s/Perl.pm//;
    $gPath =~ s/\/Skyline.pm//;
    $rcmdPath =~ s/\/Clustering.pm//;
    
    chomp($bpPath);
    chomp($gPath);
    chomp($rcmdPath);

    find({"wanted"=>\&process_g, "follow"=>1}, $gPath . '.pm');
    find({"wanted"=>\&process_g, "follow"=>1}, $gPath);
    find({"wanted"=>\&process_g, "follow"=>1}, $rcmdPath . '.pm');
    find({"wanted"=>\&process_g, "follow"=>1}, $rcmdPath);
    find({"wanted"=>\&process_bp, "follow"=>1}, $bpPath);
}


=head2 help

  Name: help   -   G-language Shell help command

  Description:
    G-language Shell help command. This command seaches through
    G-language GAE manual and BioPerl documentation (Perldoc)
    to show the corresponding entries for a given keyword.

    You can also search through the documentation with options (See below).

  Usage:
    help <options> <keyword>

    For <keyword>, you can specify G-language GAE function name
        eg. help gcskew
    or BioPerl object name with class name
        eg. Bio::Perl::get_sequence

  Options:
   -s         search through documentation with keyword
   -g         same as above, but only for G-language GAE manual
   -bp        same as above, but only for BioPerl manual

  Author: 
    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)

  History:
   20070531-01 initial posting

=cut


sub help{
    buildHelpDB() unless(scalar %pdoc);

    my $key = shift || "help";

    if ($key =~ /^\-(\S+)/){
	my $option = $1;
	$key = join(' ', @_);

	my $ans = "     Keyword \"$key\" was not found in the documentations.\n";

	my $lckey = lc($key);
	my @tmp;;
	    
	foreach my $keyword (sort keys %pdoc){
	    if ($keyword =~ /Bio/){
		next if ($option =~ 'g');
	    }else{
		next if ($option =~ 'b');
	    }

	    if(lc($pdoc{$keyword}) =~ /$lckey/){
		push(@tmp, $keyword);
	    }elsif(length($lckey) < 1){
		push(@tmp, $keyword);
	    }
	}
	    
	if(scalar(@tmp)){
	    $ans = sprintf("     Found keyword \"%s\" in the following manual pages \(%s hits\).\n\n", $key, scalar(@tmp));
	    
	    foreach my $entry (@tmp){
		my $tmp = $pdoc{$entry};

		my $abstract;
		if($tmp =~ /Name:\s*(.*)(   -   .*?)\_\_\%\%\%\%\%\_\_/){
		    $abstract = $2;
		}

		if($entry =~ /Bio::/){
		    $ans .= sprintf("              %s\n",$entry);
		}elsif($entry){
		    $ans .= sprintf("       %-30s%s\n",$entry, $abstract);
		}
	    }
	}

	print $ans;


    }else{

	my $ans = $pdoc{$key};

	if(length $ans){
	    $ans =~ s/\_\_\%\%\%\%\%\_\_/\n/g;

	    unless($ans =~ /Name: (.*)   -   (.*)\n/){
		$ans = "\n  Name: $key\n". $ans;
	    }
	}else{
	    $ans = "     Keyword: \"$key\" not found in manual.\n\n";
	    my $lckey = lc($key);
	    my @tmp;;
	    
	    foreach my $keyword (sort keys %pdoc){
		if(lc($keyword) =~ /$lckey/){
		    push(@tmp, $keyword);
		}
	    }
	    
	    unless(scalar(@tmp)){
		foreach my $keyword (sort keys %pdoc){
		    if(amatch($lckey, ['i 1'], lc($keyword))){
			push(@tmp, $keyword);
		    }
		}
	    }

	    if(scalar(@tmp)){
		$ans .= "     Did you mean:\n";
		
		foreach my $entry (@tmp){
		    $ans .= "              " . $entry . "\n";
		}
	    }

	    $ans .= "\n\n      use \"help -s\" if you want to search, or \"help help\" if desperate.\n";
	}

	print $ans;
    }
}

sub process_g{
    next unless(/\.pm$/);

    open(FILE, $File::Find::name) || die($File::Find::name, " ", $!);
    while(<FILE>){
	chomp;
	if(/^\s*\=head2\s+(.*)\s*$/){
	    my $subname = $1;

	    while(<FILE>){
		chomp;
		last if (/\=cut/);
		$pdoc{$subname} .= $_ . '__%%%%%__';
	    }
	}
    }
    close(FILE);
}


sub process_bp{
    my $class = 'Bio';

    if ($File::Find::name =~ /\/(Bio.*)\.pm$/){
	$class = $1;
	$class =~ s/\//::/g;
    }else{
	next;
    }

    open(FILE, $File::Find::name) || die($!);
    while(<FILE>){
	chomp;
	if(/^\s*\=head2\s+(\S*)\s*/){
	    my $subname = $class . '::' . $1;
	    next if ($subname =~ /Mailing Lists/ || $subname =~ /Reporting Bugs/);

	    while(<FILE>){
		chomp;
		last if (/\=cut/);
		$pdoc{$subname} .= $_ . '__%%%%%__';
	    }
	}
    }
    close(FILE);
}

1;

==> g-language-1.7.6.tar.gz: g-language-1.7.6/lib/G/Shell/Help.pm <==
#!/usr/bin/env perl

#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# This file is part of G-language Genome Analysis Environment package
#
#     Copyright (C) 2001-2007 Keio University
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# 
# G-language GAE is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
# 
# G-language GAE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with G-language GAE -- see the file COPYING.
# If not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
#END_HEADER
#
# written by Kazuharu Arakawa <gaou@sfc.keio.ac.jp> at
# G-language Project, Institute for Advanced Biosciences, Keio University.
#

package G::Shell::Help;

use strict;
use File::Find;
use String::Approx 'amatch';

require Exporter;

our @ISA = qw(Exporter);

our @EXPORT = qw(
		 help
		 buildHelpDB
	     );

#:::::::::::::::::::::::::::::::::
#       Perldoc
#:::::::::::::::::::::::::::::::::


=head1 NAME

  G::Shell::Help - G-language Shell helper module for command references

=head1 DESCRIPTION

  This class is a part of G-language Genome Analysis Environment, 
  providing functions for "help" features of the G-language Shell.

=cut

#::::::::::::::::::::::::::::::
#          Variables
#::::::::::::::::::::::::::::::

my %pdoc;

#::::::::::::::::::::::::::::::
#    Let the code begin...
#::::::::::::::::::::::::::::::


sub buildHelpDB{
    my $bpPath = qx/perldoc -l Bio::Perl/;
    my $gPath = qx/perldoc -l G::IO/;
    my $rcmdPath = qx/perldoc -l Rcmd::Clustering/;
    
    $bpPath =~ s/Perl.pm//;
    $gPath =~ s/\/IO.pm//;
    $rcmdPath =~ s/\/Clustering.pm//;
    
    chomp($bpPath);
    chomp($gPath);
    chomp($rcmdPath);

    find({"wanted"=>\&process_g, "follow"=>1}, $gPath . '.pm');
    find({"wanted"=>\&process_g, "follow"=>1}, $gPath);
    find({"wanted"=>\&process_g, "follow"=>1}, $rcmdPath . '.pm');
    find({"wanted"=>\&process_g, "follow"=>1}, $rcmdPath);
    find({"wanted"=>\&process_bp, "follow"=>1}, $bpPath);
}


=head2 help

  Name: help   -   G-language Shell help command

  Description:
    G-language Shell help command. This command seaches through
    G-language GAE manual and BioPerl documentation (Perldoc)
    to show the corresponding entries for a given keyword.

    You can also search through the documentation with options (See below).

  Usage:
    help <options> <keyword>

    For <keyword>, you can specify G-language GAE function name
        eg. help gcskew
    or BioPerl object name with class name
        eg. Bio::Perl::get_sequence

  Options:
   -s         search through documentation with keyword
   -g         same as above, but only for G-language GAE manual
   -bp        same as above, but only for BioPerl manual

  Author: 
    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)

  History:
   20070531-01 initial posting

=cut


sub help{
    buildHelpDB() unless(scalar %pdoc);

    my $key = shift || "help";

    if ($key =~ /^\-(\S+)/){
	my $option = $1;
	$key = join(' ', @_);

	my $ans = "     Keyword \"$key\" was not found in the documentations.\n";

	my $lckey = lc($key);
	my @tmp;;
	    
	foreach my $keyword (sort keys %pdoc){
	    if ($keyword =~ /Bio/){
		next if ($option =~ 'g');
	    }else{
		next if ($option =~ 'b');
	    }

	    if(lc($pdoc{$keyword}) =~ /$lckey/){
		push(@tmp, $keyword);
	    }elsif(length($lckey) < 1){
		push(@tmp, $keyword);
	    }
	}
	    
	if(scalar(@tmp)){
	    $ans = sprintf("     Found keyword \"%s\" in the following manual pages \(%s hits\).\n\n", $key, scalar(@tmp));
	    
	    foreach my $entry (@tmp){
		my $tmp = $pdoc{$entry};

		my $abstract;
		if($tmp =~ /Name:\s*(.*)(   -   .*?)\_\_\%\%\%\%\%\_\_/){
		    $abstract = $2;
		}

		if($entry =~ /Bio::/){
		    $ans .= sprintf("              %s\n",$entry);
		}elsif($entry){
		    $ans .= sprintf("       %-30s%s\n",$entry, $abstract);
		}
	    }
	}

	print $ans;


    }else{

	my $ans = $pdoc{$key};

	if(length $ans){
	    $ans =~ s/\_\_\%\%\%\%\%\_\_/\n/g;

	    unless($ans =~ /Name: (.*)   -   (.*)\n/){
		$ans = "\n  Name: $key\n". $ans;
	    }
	}else{
	    $ans = "     Keyword: \"$key\" not found in manual.\n\n";
	    my $lckey = lc($key);
	    my @tmp;;
	    
	    foreach my $keyword (sort keys %pdoc){
		if(lc($keyword) =~ /$lckey/){
		    push(@tmp, $keyword);
		}
	    }
	    
	    unless(scalar(@tmp)){
		foreach my $keyword (sort keys %pdoc){
		    if(amatch($lckey, ['i 1'], lc($keyword))){
			push(@tmp, $keyword);
		    }
		}
	    }

	    if(scalar(@tmp)){
		$ans .= "     Did you mean:\n";
		
		foreach my $entry (@tmp){
		    $ans .= "              " . $entry . "\n";
		}
	    }

	    $ans .= "\n\n      use \"help -s\" if you want to search, or \"help help\" if desperate.\n";
	}

	print $ans;
    }
}

sub process_g{
    next unless(/\.pm$/);

    open(FILE, $File::Find::name) || die($File::Find::name, " ", $!);
    while(<FILE>){
	chomp;
	if(/^\s*\=head2\s+(.*)\s*$/){
	    my $subname = $1;

	    while(<FILE>){
		chomp;
		last if (/\=cut/);
		$pdoc{$subname} .= $_ . '__%%%%%__';
	    }
	}
    }
    close(FILE);
}


sub process_bp{
    my $class = 'Bio';

    if ($File::Find::name =~ /\/(Bio.*)\.pm$/){
	$class = $1;
	$class =~ s/\//::/g;
    }else{
	next;
    }

    open(FILE, $File::Find::name) || die($!);
    while(<FILE>){
	chomp;
	if(/^\s*\=head2\s+(\S*)\s*/){
	    my $subname = $class . '::' . $1;
	    next if ($subname =~ /Mailing Lists/ || $subname =~ /Reporting Bugs/);

	    while(<FILE>){
		chomp;
		last if (/\=cut/);
		$pdoc{$subname} .= $_ . '__%%%%%__';
	    }
	}
    }
    close(FILE);
}

1;
